module define_parameters

!*************************************************************************************************************************

! Copyright  2003 S. Boragan Aruoba, Jesus Fernandez-Villaverde and Juan F. Rubio-Ramirez.

! This code is available on the World Wide Web at http://www.econ.upenn.edu/~jesusfv 
! and it is a part of the package prepared for the paper "Comparing Solution Methods for Dynamic Equilibrium Economies". 

! It may be freely reproduced for educational and research purposes, so long as it is not altered,
! this copyright notice is reproduced with it, and it is not sold for profit.

! Consent of the corresponding author (jesusfv@econ.upenn.edu) must be obtained before using all or any part of this code
! in a publication. 

! This version			: July 5, 2003

!************************************************************************************************************************	

	implicit none
	save

	! Real Kind

	integer, parameter :: real_kind = selected_real_kind(P=15)	! Ask for 15 digits of precision

	! Approximation Parameters

	integer, parameter :: n = 11	! Number of collocation points for k (roots of Chebyshev polynomial)											
	integer, parameter :: m = 9		! Number of colloation points for A (from tauchen)

	real(kind = real_kind), parameter :: eps = 1E-14		! Stopping Criterion
	real(kind = real_kind), parameter :: del = 1E-14		! Stopping Criterion
	integer, parameter :: maxit = 10000						! Maximum number of iterations

	real(kind = real_kind) :: k_min		! Boundaries of the state space for capital
	real(kind = real_kind) :: k_max 	! These will be +- some % of steady state capital

	real(kind = real_kind), parameter :: cover_k = 0.31d0	   

	real(kind = real_kind), parameter :: z_min = - 0.095d0	! These are limits where the process is sure to be inside
	real(kind = real_kind), parameter :: z_max =   0.095d0

	! Model Parameters
	
	! Utility Function
	
	real(kind = real_kind), parameter :: beta1 = 0.9896d0	! Discount rate
	real(kind = real_kind), parameter :: tau = 2.0d0		! Coefficient of Risk Aversion
	real(kind = real_kind), parameter :: theta = 0.357d0	! Power of consumption in the aggregator
	
	! Technology

	real(kind = real_kind), parameter :: alpha	= 0.4d0		! Power of capital in the production function
	real(kind = real_kind), parameter :: delta = 0.0196d0	! Depreciation

	! Shocks to Productivity (Epsilon)

	real(kind = real_kind), parameter :: rho = 0.95d0		! AR(1) parameter of the shocks
	real(kind = real_kind), parameter :: sigma = 0.007d0	! Standard deviation of epsilon	
	
	! Steady State 

	real(kind = real_kind) :: k_ss 		! Steady state capital
	real(kind = real_kind) :: l_ss		! Steady state labor
	real(kind = real_kind) :: y_ss		! Steady state output
	real(kind = real_kind) :: c_ss		! Steady state consumption

	! Misc.

	real(kind = real_kind) :: pi		! Mathematical Constant PI
	
	! Global Variables
	
	real(real_kind), dimension(m ,1) :: input_z	
	real(real_kind), dimension(m ,m) :: input_z_prime	
	real(real_kind), dimension(m ,m) :: TRANSITION
	  
	real(real_kind), allocatable, dimension(:,:) :: KK, ZZ	! States
	real(real_kind), dimension(m , m) :: ZZ_PRIME			! For integration
	real(real_kind), allocatable, dimension(:,:) :: PSI_USE
	
	real(kind = real_kind), dimension(n*m,1) :: theta_weight
	integer n_iter

end module define_parameters

