subroutine get_psi()

! This subroutine creates the global objects that carries the collocation points and PSI matrices. 

!*************************************************************************************************************************

! Copyright  2003 S. Boragan Aruoba, Jesus Fernandez-Villaverde and Juan F. Rubio-Ramirez.

! This code is available on the World Wide Web at http://www.econ.upenn.edu/~jesusfv 
! and it is a part of the package prepared for the paper "Comparing Solution Methods for Dynamic Equilibrium Economies". 

! It may be freely reproduced for educational and research purposes, so long as it is not altered,
! this copyright notice is reproduced with it, and it is not sold for profit.

! Consent of the corresponding author (jesusfv@econ.upenn.edu) must be obtained before using all or any part of this code
! in a publication. 

! This version			: July 5, 2003

!************************************************************************************************************************	

	use define_parameters

	use small_functions

	use numerical_libraries

	implicit none

	real(kind = real_kind), dimension(n_iter,1) :: k, x_n    ! From n_th order Chebyshev polynomial
	real(kind = real_kind), dimension(m,1) :: z			
	real(kind = real_kind), dimension(m,1) ::  x_m	

	real(kind = real_kind), dimension(n_iter,n_iter) :: psi_k
	real(kind = real_kind), dimension(m,m) :: psi_z
	
	real(kind = real_kind), dimension(n_iter,1)  :: ones_n 
	real(kind = real_kind), dimension(m,1)  :: ones_m

	integer count, i, j

	ones_n = 1
	ones_m = 1

	allocate(KK(n_iter * m,1))
	allocate(ZZ(n_iter * m,1))
	allocate(PSI_USE((n_iter * m) , (n_iter * m)))	

	! Roots of Chebyshev Polynomials

	! Roots of the nth order Chebyshev polynomial

	x_n = reshape( source = (/ ( (cos(pi * (2.0d0 * count -1.) / (2.0d0 * n_iter))  ), count = n_iter,1,-1) /) &
			, shape = (/ n_iter,1 /))				

	! x_n converted into capital-space
	
	k = (k_max - k_min) * (x_n + 1.0d0) / 2.0d0 + k_min      

	z = input_z
	
	x_m = reshape( source = (/ (( 2.0d0 * (z(i,1) - z_min) / (z_max - z_min) - 1.0d0 ) , i = 1, m) /), shape = (/ m, 1 /))
	
	! Create the Basis Functions for states

	psi_k = reshape( source = (/ ((cos((i-1.0d0) * acos(x_n(j,1))), i = 1, n_iter), j = 1, n_iter) /) &
			, shape = (/ n_iter,n_iter /))

	psi_z = reshape( source = (/ ((cos((i-1.0d0) * acos(x_m(j,1))), i = 1, m), j = 1, m) /) &
			, shape = (/ m,m /))
	
	! Create the Big Basis Function and States

	PSI_USE = kron_tensor(psi_k,psi_z,n_iter,n_iter,m,m)
	KK =  kron_tensor(k,ones_m,n_iter,1,m,1)
	ZZ =  kron_tensor(ones_n,Z,n_iter,1,m,1)

end subroutine