function residual(theta_in,neq)

! This function inputs the theta's and compute the residual function evaluated at the collocation points.

!*************************************************************************************************************************

! Copyright  2003 S. Boragan Aruoba, Jesus Fernandez-Villaverde and Juan F. Rubio-Ramirez.

! This code is available on the World Wide Web at http://www.econ.upenn.edu/~jesusfv 
! and it is a part of the package prepared for the paper "Comparing Solution Methods for Dynamic Equilibrium Economies". 

! It may be freely reproduced for educational and research purposes, so long as it is not altered,
! this copyright notice is reproduced with it, and it is not sold for profit.

! Consent of the corresponding author (jesusfv@econ.upenn.edu) must be obtained before using all or any part of this code
! in a publication. 

! This version			: July 5, 2003

!************************************************************************************************************************	
	
	use define_parameters

	use small_functions

	implicit none	

	!! Declare Local Variables !!

	integer neq
	real(kind = real_kind), dimension(neq,1) :: theta_in		! Weights of the Basis Functions
	real(kind = real_kind), dimension(neq,1) ::  residual
	real(kind = real_kind), dimension(neq,1) :: K_P	! Capital
	real(kind = real_kind), dimension(neq,1) :: LL	! Hours
	
	real(kind = real_kind) :: z, k, c, c_temp, l, k_prime, z_prime, x_prime_k, x_prime_z
	real(kind = real_kind) :: y, temp,c_prime, c_prime_temp, y_prime,k_double_prime, u1

	real(kind = real_kind), dimension(1,1) :: sol_integ, l_prime 

	real(kind = real_kind) :: temp_prime, u1_prime, f1_prime
		
	real(kind = real_kind), dimension(n_iter,1) :: psi_k_prime
	real(kind = real_kind), dimension(m,1) :: psi_z_prime

	real(kind = real_kind), dimension(neq,1) :: psi_prime
	real(kind = real_kind), dimension(1,m) :: integrand

	integer row,  count_int, i, j, count

	real(real_kind), dimension(1 , m ) :: TRANSITION_ROW

	real(kind = real_kind), dimension(1,m) :: temp2

	20	format(A,F6.2)   ! For printing

	if (neq /= n_iter*m) then
		print*, 'Error. n_iter * m has to be equal to neq!'
		stop
	endif

	TRANSITION_ROW(1,1:m) = TRANSITION(1,1:m)		! Since all rows are the same

	! Decision Rules	

	LL = matmul(transpose(PSI_USE),theta_in)

	! This loop computes the residual for each pair (k,a)

	row = 1

	do i = 1,n_iter
		do j = 1,m		
			
			! States	
			z = ZZ(row,1)
			k = KK(row,1)

			! Decision Rules
			
			l = LL(row,1)

			if (l < 0.1d0) l = 0.1d0
			if (l > 0.9d0) l = 0.9d0

			y = (exp(z) * (l ** (1.0d0 - alpha)) * (k ** alpha))
			
			! Using EE1

			c_temp = (theta / (1.0d0 - theta)) * (1.0d0 - l) * (1.0d0 - alpha) * (y / l)			

			k_prime = y + (1.0d0 - delta) * k - c_temp

			! Make sure k_prime is within [k_min,k_max], if not, correct. 
						
			if (k_prime < k_min) then
!				print 20, 'Warning : k_min is hit for k_prime!', k_prime	
				k_prime = k_min + 0.001d0					
				
			elseif ((k_prime > y + (1.0d0 - delta) * k) .or. (k_prime > k_max)) then
!				print 20, 'Warning : k_max is hit for k_prime!', k_prime
				k_prime = min(y + (1.0d0 - delta) * k, k_max) - 0.001d0
								
			endif

			c = y + (1.0d0 - delta) * k - k_prime
							
			if (c < 0) then
				print 20, 'Warning : c < 0 encountered!', c
				c = 10E-10											
			end if		

			x_prime_k = 2.0d0 * (k_prime - k_min) / (k_max - k_min) - 1.0d0

			psi_k_prime = reshape( source = (/ ((cos((i-1.0d0) * acos(x_prime_k))), &
							i = 1, n_iter) /) , shape = (/ n_iter,1 /))

			do count_int = 1,m

				if (TRANSITION_ROW(1,count_int) > 1e-15) then

				z_prime = ZZ_PRIME(j,count_int)

				! Construct psi_a_prime

				x_prime_z = 2.0d0 * (z_prime - z_min) / (z_max - z_min) - 1.0d0
						
				psi_z_prime = reshape( source = (/ ((cos((i-1.0d0) * acos(x_prime_z))), &
							i = 1, m) /) , shape = (/ m,1 /))

				psi_prime = kron_tensor(psi_k_prime,psi_z_prime,n_iter,1,m,1)

				l_prime = matmul(transpose(psi_prime),theta_in)

				if (l_prime(1,1) < 0.1d0) l_prime(1,1) = 0.1d0
				if (l_prime(1,1) > 0.9d0) l_prime(1,1) = 0.9d0 

				y_prime = (exp(z_prime) * (l_prime(1,1) ** (1.0d0 - alpha)) * (k_prime ** alpha))
			
				! Using EE1

				c_prime_temp = (theta / (1.0d0 - theta)) * (1.0d0 - l_prime(1,1)) * (1.0d0 - alpha) * (y_prime / l_prime(1,1))

				k_double_prime = y_prime + (1.0d0 - delta) * k_prime - c_prime_temp

				! Make sure k_double_prime is within [k_min,k_max], if not, correct. 
						
				if(k_double_prime < k_min) then
!					print 20, 'Warning : k_min is hit for k_double_prime!', k_double_prime	
					k_double_prime = k_min + 0.001d0	
						
				elseif ((k_double_prime > y_prime + (1.0d0 - delta) * k_prime) .or. (k_double_prime > k_max)) then
!					print 20, 'Warning : k_max is hit for k_double_prime!', k_double_prime
					k_double_prime = min(y_prime + (1.0d0 - delta) * k_prime, k_max) - 0.001d0
								
				endif

				c_prime =  y_prime + (1.0d0 - delta) * k_prime - k_double_prime	
				
				if (c_prime < 0) then
					print 20, 'Warning : c_prime < 0 encountered!', c_prime
					c_prime = 10E-10											
				end if 								

				temp_prime = ( (c_prime ** theta) * ( (1.0d0-l_prime(1,1)) ** (1.0d0-theta) ) ) ** (-tau)

				u1_prime = temp_prime * theta * ( (c_prime / (1.0d0-l_prime(1,1))) ** (theta - 1.0d0) )

				f1_prime = alpha * exp(z_prime) * (k_prime ** (alpha -1.0d0)) * (l_prime(1,1) ** (1.0d0 - alpha))

				integrand(1,count_int) = u1_prime * (f1_prime + 1.0d0 - delta)

				else

				integrand(1,count_int) = 0.0d0

				end if

			end do			

			sol_integ = matmul(integrand,transpose(TRANSITION_ROW))

			residual(row,1) = c -  inverse_u1(beta1 * sol_integ(1,1), l)

			row = row + 1
		end do
	end do

contains

!******************************************

function inverse_u1(in,l_in)

	! This function computes the inverse of u1, given l. 

	use define_parameters

	implicit none
	
	real(kind = real_kind) :: in, l_in, LL, P
	real(kind = real_kind) :: inverse_u1
	
	LL = (1 - l_in) ** (theta * tau - tau + 1 - theta)
	P = 1 / ( theta - 1 - theta * tau)

	inverse_u1 = ( in / (theta * LL) ) ** P

end function inverse_u1


end function residual
